﻿'''**************************************************************************
' Project  : AvionicsInstrumentControlDemo                                  
' File     : AirSpeedIndicatorInstrumentControl.cs                          
' Version  : 1                                                              
' Language : C#                                                             
' Summary  : The air speed indicator instrument control                     
' Creation : 19/06/2008                                                     
' Autor    : Guillaume CHOUTEAU                                             
' History  :                                                                
'''**************************************************************************

Imports System
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Collections
Imports System.Drawing
Imports System.Text
Imports System.Data

Namespace AvionicsInstrumentControlDemo
    Public Class AirSpeedIndicatorInstrumentControl
        Inherits InstrumentControl

#Region "Fields"

        ' Parameters
        Private airSpeed As Integer

        ' Images
        Private bmpCadran As New Bitmap(My.Resources.AirSpeedIndicator_Background)
        Private bmpNeedle As New Bitmap(My.Resources.AirSpeedNeedle)

#End Region

#Region "Contructor"

        ''' <summary>
        ''' Required designer variable.
        ''' </summary>
        Private components As System.ComponentModel.Container = Nothing

        Public Sub New()
            ' Double bufferisation
            SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
        End Sub

#End Region

#Region "Component Designer generated code"
        ''' <summary>
        ''' Required method for Designer support - do not modify 
        ''' the contents of this method with the code editor.
        ''' </summary>
        Private Overloads Sub InitializeComponent()
            components = New System.ComponentModel.Container()
        End Sub
#End Region

#Region "Paint"

        Protected Overrides Sub OnPaint(ByVal pe As PaintEventArgs)
            ' Calling the base class OnPaint
            MyBase.OnPaint(pe)

            ' Pre Display computings
            Dim ptRotation As New Point(150, 150)
            Dim ptimgNeedle As New Point(136, 39)

            bmpCadran.MakeTransparent(Color.Yellow)
            bmpNeedle.MakeTransparent(Color.Yellow)

            Dim alphaNeedle As Double = InterpolPhyToAngle(airSpeed, 0, 800, 180, 468)

            'INSTANT VB NOTE: The variable scale was renamed since Visual Basic does not handle local variables named the same as class members well:
            Dim scale_Renamed As Single = CSng(Me.Width) / bmpCadran.Width

            ' diplay mask
            Dim maskPen As New Pen(Me.BackColor, 30 * scale_Renamed)
            pe.Graphics.DrawRectangle(maskPen, 0, 0, bmpCadran.Width * scale_Renamed, bmpCadran.Height * scale_Renamed)

            ' display cadran
            pe.Graphics.DrawImage(bmpCadran, 0, 0, CSng(bmpCadran.Width * scale_Renamed), CSng(bmpCadran.Height * scale_Renamed))

            ' display small needle
            RotateImage(pe, bmpNeedle, alphaNeedle, ptimgNeedle, ptRotation, scale_Renamed)

        End Sub

#End Region

#Region "Methods"


        ''' <summary>
        ''' Define the physical value to be displayed on the indicator
        ''' </summary>
        ''' <param name="aircraftAirSpeed">The aircraft air speed in kts</param>
        Public Sub SetAirSpeedIndicatorParameters(ByVal aircraftAirSpeed As Integer)
            airSpeed = aircraftAirSpeed

            Me.Refresh()
        End Sub

#End Region

#Region "IDE"




#End Region

    End Class
End Namespace
